from komtet_kassa_linux.driver import IFptr


RECEIPT_TYPE_MAP = {
    'sell': IFptr.LIBFPTR_RT_SELL,
    'sellReturn': IFptr.LIBFPTR_RT_SELL_RETURN,
    'sellCorrection': IFptr.LIBFPTR_RT_SELL_CORRECTION,
    'sellReturnCorrection': IFptr.LIBFPTR_RT_BUY_CORRECTION,
    'buy': IFptr.LIBFPTR_RT_BUY,
    'buyReturn': IFptr.LIBFPTR_RT_BUY_RETURN,
    'buyCorrection': IFptr.LIBFPTR_RT_BUY_CORRECTION,
    # IFptr.LIBFPTR_RT_BUY_RETURN_CORRECTION
}


SNO_MAP = {
    0: IFptr.LIBFPTR_TT_OSN,
    1: IFptr.LIBFPTR_TT_USN_INCOME,
    2: IFptr.LIBFPTR_TT_USN_INCOME_OUTCOME,
    3: IFptr.LIBFPTR_TT_ENVD,
    4: IFptr.LIBFPTR_TT_ESN,
    5: IFptr.LIBFPTR_TT_PATENT
}

AGENT_TYPE_MAP = {
    'bank_payment_agent': IFptr.LIBFPTR_AT_BANK_PAYING_AGENT,
    'bank_payment_subagent': IFptr.LIBFPTR_AT_BANK_PAYING_SUBAGENT,
    'payment_agent': IFptr.LIBFPTR_AT_PAYING_AGENT,
    'payment_subagent': IFptr.LIBFPTR_AT_PAYING_SUBAGENT,
    'solicitor': IFptr.LIBFPTR_AT_ATTORNEY,
    'commissionaire': IFptr.LIBFPTR_AT_COMMISSION_AGENT,
    'agent': IFptr.LIBFPTR_AT_ANOTHER
}

TAX_MAP = {
    # IFptr.LIBFPTR_TAX_DEPARTMENT,
    '20': IFptr.LIBFPTR_TAX_VAT20,
    '18': IFptr.LIBFPTR_TAX_VAT18,
    '10': IFptr.LIBFPTR_TAX_VAT10,
    '120': IFptr.LIBFPTR_TAX_VAT120,
    '118': IFptr.LIBFPTR_TAX_VAT118,
    '110': IFptr.LIBFPTR_TAX_VAT110,
    '0': IFptr.LIBFPTR_TAX_VAT0,
    'no': IFptr.LIBFPTR_TAX_NO
}

PAYMENT_MAP = {
    'cash': IFptr.LIBFPTR_PT_CASH,
    'card': IFptr.LIBFPTR_PT_ELECTRONICALLY,
    'prepayment': IFptr.LIBFPTR_PT_PREPAID,
    'credit': IFptr.LIBFPTR_PT_CREDIT,
    'counter_provisioning': IFptr.LIBFPTR_PT_OTHER
}


CORRECTION_RECEIPT_BASIS_MAP = {
    "self": 0,
    "forced": 1
}

PAYMENT_METHOD_MAP = {
    'pre_payment_full': 1,
    'pre_payment_part': 2,
    'advance': 3,
    'full_payment': 4,
    'credit_part': 5,
    'credit_pay': 6,
    'credit': 7
}

PAYMENT_OBJECT_MAP = {
    'product': 1,
    'product_practical': 2,
    'work': 3,
    'service': 4,
    'gambling_bet': 5,
    'gambling_win': 6,
    'lottery_bet': 7,
    'lottery_win': 8,
    'rid': 9,
    'payment': 10,
    'commission': 11,
    'composite': 12,
    'other': 13
}


class Agent:

    def __init__(self, kkt, type):
        self.kkt = kkt
        self.params = {1222: AGENT_TYPE_MAP[type]}

    def set_paying_agent(self, operation, phones):
        ''' Установка атрибутов платежного агента '''
        self.kkt.fptr.setParam(1044, operation)
        for phone in phones:
            self.kkt.fptr.setParam(1073, phone)

        self.kkt.fptr.utilFormTlv()
        self.params[1223] = self.kkt.fptr.getParamByteArray(IFptr.LIBFPTR_PARAM_TAG_VALUE)

    def set_receive_payments_operator(self, phones):
        ''' Установка атрибутов оператора по приему платежей '''
        for phone in phones:
            self.kkt.fptr.setParam(1074, phone)

        self.kkt.fptr.utilFormTlv()
        self.params[1223] = self.kkt.fptr.getParamByteArray(IFptr.LIBFPTR_PARAM_TAG_VALUE)

    def set_money_transfer_operator(self, phones, name, address, inn):
        ''' Установка атрибутов оператора перевода '''
        for phone in phones:
            self.kkt.fptr.setParam(1075, phone)
        self.kkt.fptr.setParam(1026, name)
        self.kkt.fptr.setParam(1005, address)
        self.kkt.fptr.setParam(1016, inn)

        self.kkt.fptr.utilFormTlv()
        self.params[1223] = self.kkt.fptr.getParamByteArray(IFptr.LIBFPTR_PARAM_TAG_VALUE)

    def serialize(self):
        return self.params


class Supplier:

    def __init__(self, kkt, phones, name, inn):
        self.kkt = kkt
        for phone in phones:
            self.kkt.fptr.setParam(1171, phone)
        self.kkt.fptr.setParam(1225, name)
        self.kkt.fptr.utilFormTlv()
        self.params = {
            1224: self.kkt.fptr.getParamByteArray(IFptr.LIBFPTR_PARAM_TAG_VALUE),
            1226: inn
        }

    def serialize(self):
        return self.params


class Receipt:

    def __init__(self, kkt, type='sell'):
        self.kkt = kkt
        self.params = {
            IFptr.LIBFPTR_PARAM_RECEIPT_TYPE: RECEIPT_TYPE_MAP[type],
            1055: IFptr.LIBFPTR_TT_DEFAULT
        }
        self.positions = []
        self.payments = []
        self.cashier = {}

    def _set_param(self, key, val):
        if not val:
            if key in self.params:
                del self.params[key]
            return

        self.params[key] = val

    @property
    def sno(self):
        return self.params[1055]

    @sno.setter
    def sno(self, val):
        self.params[1055] = SNO_MAP[val]

    @property
    def email(self):
        return self.params[1008]

    @email.setter
    def email(self, val):
        self._set_param(1008, val)

    @property
    def payment_address(self):
        return self.params.get(1187)

    @payment_address.setter
    def payment_address(self, val):
        self._set_param(1187, val)

    def set_cashier(self, name, inn):
        self.cashier[1021] = name
        self.cashier[1203] = inn

    def set_client(self, inn=None, name=None):
        self._set_param(1228, inn)
        self._set_param(1227, name)

    def add_position(self, name, price, quantity, total, vat, measurement_unit=None,
                     payment_method=None, payment_object=None, agent=None, supplier=None):
        params = {
            IFptr.LIBFPTR_PARAM_COMMODITY_NAME: name,
            IFptr.LIBFPTR_PARAM_PRICE: price,
            IFptr.LIBFPTR_PARAM_QUANTITY: quantity,
            IFptr.LIBFPTR_PARAM_POSITION_SUM: total,
            IFptr.LIBFPTR_PARAM_TAX_TYPE: TAX_MAP[vat],
        }
        if measurement_unit:
            params[1197] = measurement_unit
        if payment_method:
            params[1214] = PAYMENT_METHOD_MAP[payment_method]
        if payment_object:
            params[1212] = PAYMENT_OBJECT_MAP[payment_object]

        if agent:
            params.update(agent.serialize())

        if supplier:
            params.update(supplier.serialize())

        self.positions.append(params)

    def add_payment(self, sum, type=None):
        self.payments.append({
            IFptr.LIBFPTR_PARAM_PAYMENT_TYPE: PAYMENT_MAP.get(type,
                                                              IFptr.LIBFPTR_PT_ELECTRONICALLY),
            IFptr.LIBFPTR_PARAM_PAYMENT_SUM: sum
        })

    def set_correction_info(self, basis, name, date, number):
        self.kkt.fptr.setParam(1177, name)
        self.kkt.fptr.setParam(1178, date)
        self.kkt.fptr.setParam(1179, number)
        self._check(self.kkt.fptr.utilFormTlv())
        self.params[1174] = self.kkt.fptr.getParamByteArray(IFptr.LIBFPTR_PARAM_TAG_VALUE)
        self.params[1173] = CORRECTION_RECEIPT_BASIS_MAP[basis]

    def _check(self, res):
        return self.kkt._check(res)

    def cansel(self):
        self._check(self.kkt.fptr.cancelReceipt())

    def close(self):
        self._check(self.kkt.fptr.closeReceipt())

    def fiscalize(self, is_print=True):
        def set_params(params):
            for param, value in params.items():
                self.kkt.fptr.setParam(param, value)

        if self.cashier:
            set_params(self.cashier)
            self._check(self.kkt.fptr.operatorLogin())

        set_params(self.params)
        self.kkt.fptr.setParam(IFptr.LIBFPTR_PARAM_RECEIPT_ELECTRONICALLY, not is_print)
        self._check(self.kkt.fptr.openReceipt())

        for position in self.positions:
            set_params(position)
            self._check(self.kkt.fptr.registration())

        for payment in self.payments:
            set_params(payment)
            self._check(self.kkt.fptr.payment())

        self.kkt.fptr.setParam(IFptr.LIBFPTR_PARAM_SUM,
                               sum(map(lambda p: p[IFptr.LIBFPTR_PARAM_PAYMENT_SUM],
                                       self.payments), 0))
        self.kkt.fptr.receiptTotal()

        self._check(self.kkt.fptr.closeReceipt())

        self._check(self.kkt.fptr.checkDocumentClosed())
        if not self.kkt.fptr.getParamBool(IFptr.LIBFPTR_PARAM_DOCUMENT_CLOSED):
            self.kkt.fptr.cancelReceipt()
            return

        fiscal_data = self.kkt.get_last_receipt()
        fiscal_data.update(self.kkt.get_shift_info())
        return fiscal_data

    def print_out(self):
        return self.fiscalize(False)
