import logging.config
import os.path
from configparser import ConfigParser

from komtet_kassa_linux.libs import komtet_kassa
from komtet_kassa_linux.models.meta import configure as db_configure

from .manager import KmManager


logger = logging.getLogger(__name__)


CONFIG_FILE = 'rkm.ini'

CONFIG = {
    'db_file': os.path.join(os.path.expanduser('~'), 'rkm', 'rkm.db'),
    'api_url': 'https://kassa.komtet.ru/api/pos/v1',
    'rent_station': None
}
CONFIG['db_url'] = 'sqlite:///' + CONFIG['db_file']

if os.path.exists(CONFIG_FILE):
    config_parser = ConfigParser()
    config_parser.read(CONFIG_FILE)
    if 'rkm_manager' in config_parser.sections():
        CONFIG.update(config_parser['rkm_manager'])

    logging.config.fileConfig(config_parser)


db_configure(CONFIG)
komtet_kassa.KOMTET_KASSA_API = CONFIG['api_url']

km_manager = KmManager(CONFIG['db_file'], rent_station=CONFIG['rent_station'])


def main():
    logger.info('Start main loop')
    try:
        km_manager.loop()
    except Exception:
        logger.exception('Error in main loop')
    else:
        logger.info('Stop main loop')
