import logging
import os

from pyudev import Context, Monitor, MonitorObserver
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer

from komtet_kassa_linux.libs.helpers import ATOL_MODEL_IDS, ATOL_VENDOR_IDS


logger = logging.getLogger(__name__)


class DeviceObserver(MonitorObserver):

    def __init__(self, connect_handler=None, disconnect_handler=None):
        self._connect_handler = connect_handler
        self._disconnect_handler = disconnect_handler

        monitor = Monitor.from_netlink(Context())
        monitor.filter_by(subsystem='tty')
        super(DeviceObserver, self).__init__(monitor, self.__event)

    def __event(self, action, device):
        if (
            device['ID_VENDOR_ID'] in ATOL_VENDOR_IDS and
            device['ID_MODEL_ID'] in ATOL_MODEL_IDS and
            device['ID_USB_INTERFACE_NUM'] == '00'
        ):

            if action == 'add':
                logger.info('add - {}'.format(device['DEVNAME']))
                self._connect_handler(device['DEVNAME'])
            elif action == 'remove':
                logger.info('remove - {}'.format(device['DEVNAME']))
                self._disconnect_handler(device['DEVNAME'])


class FileObserver(Observer):

    def __init__(self, file_path, modify_handler):
        super(FileObserver, self).__init__()
        self.schedule(WatchdogHandler(file_path.split('/')[-1], modify_handler),
                      path=os.path.dirname(file_path) or '.')


class WatchdogHandler(FileSystemEventHandler):

    def __init__(self, filename, modify_handler):
        self.filename = filename
        self.modify_handler = modify_handler

    def on_modified(self, event):
        if event.src_path.endswith(self.filename):
            self.modify_handler()
