import logging.config
import os
from configparser import ConfigParser

import requests
from flask import Flask
from sqlalchemy import event
from sqlalchemy.exc import DatabaseError

from komtet_kassa_linux.libs import komtet_kassa
from komtet_kassa_linux.models import session
from komtet_kassa_linux.models.meta import configure as db_configure


CONFIG_FILE = 'rkm.ini'

CONFIG = {
    'db_file': os.path.join(os.path.expanduser('~'), 'rkm', 'rkm.db'),
    'api_url': 'https://kassa.komtet.ru/api/pos/v1',
    'rent_station': None,
    'latest_version_info': 'https://dist-kassa.komtet.ru/products/KOMTETKassaLinux/latest.json',
    'secret_key': 'IjI2MjViNDkwYmFkNjAxZmY0NTVjZWQ5MmRlNGY5Yjk3ODgw'
}
CONFIG['db_url'] = 'sqlite:///' + CONFIG['db_file']

if os.path.exists(CONFIG_FILE):
    config_parser = ConfigParser()
    config_parser.read(CONFIG_FILE)
    if 'rkm_web' in config_parser.sections():
        CONFIG.update(config_parser['rkm_web'])

    logging.config.fileConfig(config_parser)

logger = logging.getLogger(__name__)

app = Flask('raspberry_km',
            template_folder=os.path.join(os.path.dirname(__file__), 'templates'),
            static_folder=os.path.join(os.path.dirname(__file__), 'static'))
app.config['is_lease'] = bool(CONFIG['rent_station'])
app.secret_key = CONFIG['secret_key']

from komtet_kassa_linux.web import routes  # isort:skip

db_configure(CONFIG)
komtet_kassa.KOMTET_KASSA_API = CONFIG['api_url']


@app.teardown_appcontext
def close_db(error):
    session.bind.connect().close()


@app.after_request
def session_commit(response):
    if response.status_code < 400:
        try:
            session.commit()
        except DatabaseError:
            session.remove()
            raise

    return response


# Отправка дампа БД после её изменения
@event.listens_for(session, 'before_commit')
def send_dump(session):
    if CONFIG.get('dump_server'):
        with open(CONFIG['db_file'], 'rb') as ofile:
            response = requests.post(CONFIG['dump_server'], files={'file': ofile})
            response.raise_for_status()
            logger.info('DB is sent to dump server %s', CONFIG['dump_server'])


def main():
    app.run()
