import os
import shutil
from unittest.mock import MagicMock, patch

import pytest
from sqlalchemy import create_engine

from komtet_kassa_linux.devices import KKT, Shift
from komtet_kassa_linux.km_manager.km import STORE_DIR
from komtet_kassa_linux.models.meta import (BaseModel as Base, configure as db_configure,
                                            session as DBSession)
from komtet_kassa_linux.web import app


def pytest_addoption(parser):
    parser.addoption('--all', default=False)


@pytest.fixture(autouse=True)
def store():
    yield
    shutil.rmtree(STORE_DIR, ignore_errors=True)
    os.makedirs(STORE_DIR, exist_ok=True)


@pytest.fixture()
def Fptr():
    yield MagicMock()


@pytest.fixture()
def kkt(Fptr):
    with patch('komtet_kassa_linux.devices.kkt.IFptr', return_value=Fptr):
        yield KKT('/dev/ttyACM0')


@pytest.fixture()
def shift(kkt):
    yield Shift(kkt)


@pytest.fixture
def client():
    app.config['TESTING'] = True
    app.config['SECRET_KEY'] = 'secret test key'

    client = app.test_client()

    yield client


@pytest.fixture()
def engine():
    path = 'test_rkm.db'

    yield create_engine('sqlite:///' + path)
    os.remove(path)


@pytest.fixture
def tables(engine):
    Base.metadata.create_all(engine)
    yield
    Base.metadata.drop_all(engine)


@pytest.fixture
def dbsession(engine, tables):
    session = DBSession(bind=engine)

    yield session

    DBSession.remove()
    session.close()
